package com.ibm.oti.shared;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

import java.net.URL;
import java.security.AccessController;
import java.util.HashMap;

import com.ibm.oti.util.PriviAction;

/**
 * Implementation of SharedClassHelperFactory.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedClassHelperFactory
 * @see SharedAbstractHelperFactory
 */
public class SharedClassHelperFactoryImpl extends SharedAbstractHelperFactory implements SharedClassHelperFactory {

	static HashMap helpers;
	private static SharedClassFilter globalSharingFilter;
	private static final String GLOBAL_SHARING_FILTER = "com.ibm.oti.shared.SharedClassGlobalFilterClass";

	static {
		helpers = new HashMap();
	}

	private static SharedClassFilter getGlobalSharingFilter() {
		if (globalSharingFilter == null) {
			try {
				String className = (String)AccessController.doPrivileged(new PriviAction(GLOBAL_SHARING_FILTER));
				if(null != className) {
					Class<SharedClassFilter> filterClass = (Class<SharedClassFilter>)Class.forName(className);
					if (filterClass != null) {
						globalSharingFilter = filterClass.newInstance();
					}
				}
			} catch (Exception e) {
				return null;
			}
		}
		return globalSharingFilter;
	}

	public SharedClassHelper findHelperForClassLoader(ClassLoader loader) {
		return (SharedClassHelper)findSharedHelperForClassLoader(loader);
	}

	public SharedClassTokenHelper getTokenHelper(
			ClassLoader loader, SharedClassFilter filter) throws HelperAlreadyDefinedException {
		SharedClassTokenHelper helper = getTokenHelper(loader);
		if (helper!=null) helper.setSharingFilter(filter);
		return helper;
	}

	public SharedClassTokenHelper getTokenHelper(ClassLoader loader) throws HelperAlreadyDefinedException {
		if (loader==null) {
			return null;
		}
		synchronized(helpers) {
			SharedClassHelper helper = findHelperForClassLoader(loader);

			if (helper!=null) {
				if (helper instanceof SharedClassTokenHelper) {
					return (SharedClassTokenHelper)helper;
				}
				throw new HelperAlreadyDefinedException("A different type of helper already exists for this classloader");
			} else {
				boolean canFind = canFind(loader);
				boolean canStore = canStore(loader);

				if (canFind || canStore) {
					SharedClassTokenHelper result = new SharedClassTokenHelperImpl(loader, getNewID(), canFind, canStore);
					if (result != null) {
						SharedClassFilter filter = getGlobalSharingFilter();
						if (filter != null) {
							result.setSharingFilter(filter);
						}
					}
					addHelper(loader, result);
					return result;
				}
			}
		}
		return null;
	}

	public SharedClassURLHelper getURLHelper(
			ClassLoader loader, SharedClassFilter filter) throws HelperAlreadyDefinedException {
		SharedClassURLHelper helper = getURLHelper(loader);
		if (helper!=null) helper.setSharingFilter(filter);
		return helper;
	}

	public SharedClassURLHelper getURLHelper(ClassLoader loader) throws HelperAlreadyDefinedException {
		if (loader==null) {
			return null;
		}
		synchronized(helpers) {
			SharedClassHelper helper = findHelperForClassLoader(loader);

			if (helper!=null) {
				if (helper instanceof SharedClassURLHelper) {
					return (SharedClassURLHelper)helper;
				}
				throw new HelperAlreadyDefinedException("A different type of helper already exists for this classloader");
			} else {
				boolean canFind = canFind(loader);
				boolean canStore = canStore(loader);

				if (canFind || canStore) {
					SharedClassURLHelper result = new SharedClassURLHelperImpl(loader, getNewID(), canFind, canStore);
					if (result != null) {
						SharedClassFilter filter = getGlobalSharingFilter();
						if (filter != null) {
							result.setSharingFilter(filter);
						}
					}
					addHelper(loader, result);
					return result;
				}
			}
		}
		return null;
	}

	public SharedClassURLClasspathHelper getURLClasspathHelper(
			ClassLoader loader, URL[] classpath, SharedClassFilter filter) throws HelperAlreadyDefinedException {
		SharedClassURLClasspathHelper helper = getURLClasspathHelper(loader,classpath);
		if (helper!=null) helper.setSharingFilter(filter);
		return helper;
	}

	public SharedClassURLClasspathHelper getURLClasspathHelper(
			ClassLoader loader, URL[] classpath)  throws HelperAlreadyDefinedException {
		if (loader==null || classpath==null) {
			return null;
		}
		synchronized(helpers) {
			SharedClassHelper helper = findHelperForClassLoader(loader);
			SharedClassURLClasspathHelperImpl result;
			boolean found = true;

			if (helper!=null) {
				if (helper instanceof SharedClassURLClasspathHelper) {
					result = (SharedClassURLClasspathHelperImpl)helper;
					URL[] testCP = result.getClasspath();
					for (int j=0; j<classpath.length; j++) {
						if (!classpath[j].equals(testCP[j])) {
							found = false;
							break;
						}
					}
					if (found) {
						return result;
					} else {
						throw new HelperAlreadyDefinedException("A SharedClassURLClasspathHelper already exists for this classloader with a different classpath");
					}
				} else {
					throw new HelperAlreadyDefinedException("A different type of helper already exists for this classloader");
				}
			} else {
				boolean canFind = canFind(loader);
				boolean canStore = canStore(loader);

				if (canFind || canStore) {
					result = new SharedClassURLClasspathHelperImpl(loader, classpath, getNewID(), canFind, canStore);
					if (result != null) {
						SharedClassFilter filter = getGlobalSharingFilter();
						if (filter != null) {
							result.setSharingFilter(filter);
						}
					}
					addHelper(loader, result);
					return result;
				}
			}
		}
		return null;
	}

	HashMap getHelperMap() {
		return helpers;
	}

}
